/*
 * mms-appl.h - interface to application layer
 */

#ifndef MMS_APPL_H
#define MMS_APPL_H

#include "wap/wap.h"
#include "wap/timers.h"
#include "mms/mms_pdu.h"

#define state_idle 0
#define state_connecting 1
#define state_retrieving 2
#define state_waiting 3

#define BEARER_POLLING_INTERVAL 5

#define MAX_BEARER_DELAY 300

struct mmsclientmachine {
    int state;
    Timer *timer;
    long machine_id;
    long transaction_id;
    WAPAddrTuple *addrtuple;
    MMS_PDU *notification;
    FILE *savedNotification;
    MMS_PDU *retrieveconf;
    Octstr *filename;
};


static enum {
    idle, waiting
} clientsendmachinestates;


struct mmsclientsendmachine {
    int state;
    Timer *timer;
    long machine_id;
    long transaction_id;
    WAPEvent *mms_send_req;
    MMS_PDU *sendconf;
    WAPAddrTuple *addrtuple;
    HTTPClient *client;
};



typedef struct mmsclientsendmachine MMSClientSendMachine;
typedef struct mmsclientmachine MMSClientMachine;

MMSClientSendMachine *mms_create_client_send_machine(WAPEvent *event);
MMSClientMachine *mms_create_client_machine(MMS_PDU *pdu);
void mms_destroy_client_send_machine(MMSClientSendMachine *machine);
void mms_destroy_client_machine(MMSClientMachine *machine);
void *remember_receiver(unsigned long handle, int method, Octstr *url, List *headers, Octstr *body, unsigned int retries);


void mms_appl_init(Cfg *cfg);
void mms_appl_dispatch(WAPEvent *event);
void mms_appl_shutdown(void);

#endif
